<?php
// Short Idea Note HTTP server script
// (c) 2014 TNK Software / Tanaka Yusuke

// NOTICE : This sample file doesn't consider security. 
//          Please implement compulsion of HTTPS communication, 
//          permission of the database file, etc on your environment.

function handleError($errno, $errstr, $errfile, $errline, array $errcontext)
{
	if (error_reporting() === 0) return false;
	throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
}
set_error_handler("handleError");

$res = array();

function printResult(){
	$pm = func_get_args();
	
	$success = $pm[0];
	if(!isset($pm[1])){
		$msg = ($success === true) ? "success" : "An unknown error occured.";
	}else{
		$msg = $pm[1];
	}
	
	global $res;
	$res["success"] = $success;
	$res["message"] = $msg;
	header("Content-Type: text/javascript; charset=utf-8");
	echo json_encode($res);
	exit();
}

$data = json_decode(file_get_contents('php://input'), TRUE);
if(isset($data["locale"])){
	$loc = $data["locale"];
}else{
	$loc = "en-us";
}

if(!isset($data["accesskey"]) || $data["accesskey"]!="shortideanote"){
	printResult(false, "Wrong access key");
}else if(isset($data["method"]) == false){
	printResult(false, "\"method\" not defined.");
}else{
	$db = new PDO("sqlite:sample.db");
	$db->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);

	// This is the method list for Ver 1.0
	// Usually, it is effective only by executing an recieved SQL query.

	// initialize : First connection from ShortIdeaNote
	// createcard : Create a card table
	// createoption : Create a setting table
	// setdbname : Update a database name
	// cardcount : Get card count
	// getcard : Select a row from card table by index(*need idx field)
	// insertcard Insert a row into card table
	// updatecard Update a row from card table by id
	// deletecard : Delete rows from card table by indexes
	// getcandicate : Select word group from card table
	// getallrows : Select all rows from card
	// getrows : Select rows from card table by ids
	// search : Select rows from card table by the user's custom search
	// quicksearch : Select a row from card table by the user's custom search
	// savesearch : Replace a row into search table by the name
	// deletesearch : Delete a row from search table by the name
	// enumsearch : Select all rows from search table
	// loadsearch : Select a row from search table by the name
	// replace : Update rows from card table by the key and ids
	// replaceword : Update rows from card table with REPLACE() function
	// chamgetemplate : Change a template

	try{
		$method = $data["method"];
		if($method == "connect"){
			// Returns initialization data
			$res["getbysql"] = true;	// Get the query data by SQL(based on SQLite)
			$res["getbyraw"] = false;	// Get the query data by the software raw data
			$res["permission"] = array(
				"insert"=>true,	// Allow insert query
				"update"=>true,	// Allow update query
				"delete"=>true,	// Allow delete query
				"search"=>true,	// Allow saving search data into this db
				"modify"=>true	// Allow change a template
			);

			$s = $db->query("select * from setting");
			if($s === false){
				// Get a table creation query from ShortIdeaNote
				$res["createnew"] = true;
				printResult(true, "Need creation data");
			}else{
				// If a database exists, returns database setting
				$res["createnew"] = false;
				$res["rows"] = $s->fetchAll();
			}
		}else{
			$s = false;
			foreach($data["commands"] as $c){
				$sql = $c["sql"];
				if(isset($c["params"])){
					$s = $db->prepare($sql);
					if($s === false) continue;

					$pms = $c["params"];
					$pc = count($pms);
					for($i = 0; $i<$pc;$i++){
						$s->bindValue($i+1, $pms[$i], is_int($pms[$i]) ? PDO::PARAM_INT : PDO::PARAM_STR);
					}
					$s->execute();
				}else{
					$s = $db->query($sql);
				}
			}
			if($s !== false) $res["rows"] = $s->fetchAll();
		}
		printResult(true);
	} catch (Exception $ex) {
		printResult(false, $ex->getMessage());
	}
}
